@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2020, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,6 +44,7 @@
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "sql/current_thd.h"
+#include "sql/protocol.h"
 #include "sql/psi_memory_key.h"
 #include "sql/query_options.h"
 #include "sql/rpl_context.h"
@@ -61,6 +62,10 @@
 
 static void store_lenenc_string(String &to, const char *from, size_t length);
 
+static bool is_component_registered_var_name(const char *name) {
+  return strchr(name, '.') != nullptr;
+}
+
 /**
   Session_sysvars_tracker
   -----------------------
@@ -181,7 +186,7 @@ class Session_sysvars_tracker : public State_tracker {
   bool check(THD *thd, set_var *var) override;
   bool update(THD *thd) override;
   bool store(THD *thd, String &buf) override;
-  void mark_as_changed(THD *thd, LEX_CSTRING *tracked_item_name) override;
+  void mark_as_changed(THD *thd, LEX_CSTRING tracked_item_name) override;
   /* callback */
   static const uchar *sysvars_get_key(const uchar *entry, size_t *length);
 
@@ -211,7 +216,7 @@ class Current_schema_tracker : public State_tracker {
   bool check(THD *, set_var *) override { return false; }
   bool update(THD *thd) override;
   bool store(THD *thd, String &buf) override;
-  void mark_as_changed(THD *thd, LEX_CSTRING *tracked_item_name) override;
+  void mark_as_changed(THD *thd, LEX_CSTRING tracked_item_name) override;
 };
 
 /* To be used in expanding the buffer. */
@@ -233,8 +238,8 @@ static const unsigned int EXTRA_ALLOC = 1024;
 */
 class Session_gtids_ctx_encoder {
  public:
-  Session_gtids_ctx_encoder() {}
-  virtual ~Session_gtids_ctx_encoder() {}
+  Session_gtids_ctx_encoder() = default;
+  virtual ~Session_gtids_ctx_encoder() = default;
 
   /*
    This function SHALL encode the collected GTIDs into the buffer.
@@ -263,8 +268,8 @@ class Session_gtids_ctx_encoder {
 
 class Session_gtids_ctx_encoder_string : public Session_gtids_ctx_encoder {
  public:
-  Session_gtids_ctx_encoder_string() {}
-  ~Session_gtids_ctx_encoder_string() override {}
+  Session_gtids_ctx_encoder_string() = default;
+  ~Session_gtids_ctx_encoder_string() override = default;
 
   ulonglong encoding_specification() override { return 0; }
 
@@ -361,11 +366,11 @@ class Session_gtids_tracker
   bool check(THD *, set_var *) override { return false; }
   bool update(THD *thd) override;
   bool store(THD *thd, String &buf) override;
-  void mark_as_changed(THD *thd, LEX_CSTRING *tracked_item_name) override;
+  void mark_as_changed(THD *thd, LEX_CSTRING tracked_item_name) override;
 
   // implementation of the Session_gtids_ctx::Ctx_change_listener
   void notify_session_gtids_ctx_change() override {
-    mark_as_changed(nullptr, nullptr);
+    mark_as_changed(nullptr, {});
   }
 };
 
@@ -382,7 +387,7 @@ void Session_sysvars_tracker::vars_list::reset() {
   the members from the other.
 
   @@param  from    Source vars_list object.
-  @@param  thd     THD handle to retrive the charset in use.
+  @@param  thd     THD handle to retrieve the charset in use.
 
   @@return    true if the m_registered_sysvars hash has any records.
               Else the value of track_all.
@@ -463,7 +468,6 @@ bool Session_sysvars_tracker::vars_list::parse_var_list(
     THD *thd, LEX_STRING var_list, bool throw_error,
     const CHARSET_INFO *char_set, bool session_created) {
   const char *separator = ",";
-  char *token, *lasts = nullptr; /* strtok_r */
 
   if (!var_list.str) {
     variables_list = nullptr;
@@ -483,52 +487,98 @@ bool Session_sysvars_tracker::vars_list::parse_var_list(
     }
   }
 
-  token = my_strtok_r(variables_list, separator, &lasts);
+  bool needs_system_variable_hash_lock = false;
+  bool needs_plugin_lock = false;
+  if (thd == nullptr || session_created) {
+    for (char *lasts, *token = my_strtok_r(variables_list, separator, &lasts);
+         token != nullptr; token = my_strtok_r(nullptr, separator, &lasts)) {
+      LEX_STRING var{token, strlen(token)};
+      /* Remove leading/trailing whitespace. */
+      trim_whitespace(char_set, &var);
+      if (var.length == 0) {
+        continue;
+      }
+      switch (System_variable_tracker::make_tracker(to_string_view(var))
+                  .lifetime()) {
+        case System_variable_tracker::PLUGIN:
+          needs_plugin_lock = true;
+          needs_system_variable_hash_lock = true;
+          break;
+        case System_variable_tracker::COMPONENT:
+          needs_system_variable_hash_lock = true;
+          break;
+        default:
+          break;
+      }
+      if (needs_plugin_lock && needs_system_variable_hash_lock) {
+        break;
+      }
+    }
+  }
+  strncpy(variables_list, var_list.str, var_list.length);
+
+  struct RAII {
+    RAII(bool needs_system_variable_hash_lock, bool needs_plugin_lock)
+        : m_needs_system_variable_hash_lock{needs_system_variable_hash_lock},
+          m_needs_plugin_lock{needs_plugin_lock} {
+      extern mysql_mutex_t LOCK_plugin;
+      extern mysql_rwlock_t LOCK_system_variables_hash;
+      if (m_needs_system_variable_hash_lock) {
+        mysql_mutex_assert_not_owner(&LOCK_plugin);
+        mysql_rwlock_rdlock(&LOCK_system_variables_hash);
+      }
+      if (m_needs_plugin_lock) {
+        mysql_mutex_lock(&LOCK_plugin);
+      }
+    }
+    ~RAII() {
+      if (m_needs_plugin_lock) {
+        extern mysql_mutex_t LOCK_plugin;
+        mysql_mutex_unlock(&LOCK_plugin);
+      }
+      if (m_needs_system_variable_hash_lock) {
+        extern mysql_rwlock_t LOCK_system_variables_hash;
+        mysql_rwlock_unlock(&LOCK_system_variables_hash);
+      }
+    }
+    const bool m_needs_system_variable_hash_lock;
+    const bool m_needs_plugin_lock;
+  } raii{needs_system_variable_hash_lock, needs_plugin_lock};
 
   track_all = false;
-  /*
-    If Lock to the plugin mutex is not acquired here itself, it results
-    in having to acquire it multiple times in find_sys_var_ex for each
-    token value. Hence the mutex is handled here to avoid a performance
-    overhead.
-  */
-  if (!thd || session_created) lock_plugin_mutex();
-  while (token) {
-    LEX_STRING var;
-    var.str = token;
-    var.length = strlen(token);
 
+  for (char *lasts, *token = my_strtok_r(variables_list, separator, &lasts);
+       token != nullptr; token = my_strtok_r(nullptr, separator, &lasts)) {
+    LEX_STRING var{token, strlen(token)};
     /* Remove leading/trailing whitespace. */
     trim_whitespace(char_set, &var);
+    if (var.length == 0) {
+      continue;
+    }
 
     if (!thd || session_created) {
-      if (find_sys_var_ex(thd, var.str, var.length, throw_error, true)) {
+      if (!System_variable_tracker::make_tracker(to_string_view(var))
+               .access_system_variable(
+                   thd, {}, Suppress_not_found_error::YES,
+                   Force_sensitive_system_variable_access::NO,
+                   Is_already_locked::YES)) {
         if (insert(nullptr, to_lex_cstring(var)) == true) {
-          /* Error inserting into the hash. */
-          unlock_plugin_mutex();
-          return true; /* Error */
+          return true;  // Error inserting into the hash.
         }
-      }
-
-      else if (throw_error) {
-        DBUG_ASSERT(thd);
+      } else if (throw_error) {
+        assert(thd);
         push_warning_printf(
             thd, Sql_condition::SL_WARNING, ER_WRONG_VALUE_FOR_VAR,
             "%s is not a valid system variable and will be ignored.", token);
       } else {
-        unlock_plugin_mutex();
         return true;
       }
     } else {
       if (insert(nullptr, to_lex_cstring(var)) == true) {
-        /* Error inserting into the hash. */
-        return true; /* Error */
+        return true;  // Error inserting into the hash.
       }
     }
-
-    token = my_strtok_r(nullptr, separator, &lasts);
   }
-  if (!thd || session_created) unlock_plugin_mutex();
 
   return false;
 }
@@ -626,13 +676,7 @@ bool Session_sysvars_tracker::update(THD *thd) {
 */
 
 bool Session_sysvars_tracker::store(THD *thd, String &buf) {
-  char val_buf[1024];
-  const char *value;
   SHOW_VAR *show;
-  sys_var *var;
-  const CHARSET_INFO *charset;
-  size_t val_length, length;
-  uchar *to;
 
   if (!(show = (SHOW_VAR *)thd->alloc(sizeof(SHOW_VAR)))) return true;
 
@@ -654,20 +698,35 @@ bool Session_sysvars_tracker::store(THD *thd, String &buf) {
             });
 
   for (sysvar_node_st *node : vars) {
-    if (node->m_changed &&
-        (var = find_sys_var_ex(thd, node->m_sysvar_name.str,
-                               node->m_sysvar_name.length, true, false))) {
+    if (!node->m_changed) {
+      continue;
+    }
+    if (is_component_registered_var_name(node->m_sysvar_name.str)) {
+      /*
+        The current system variable is a component-registered one.
+        Currently, all component variables are "global" (i.e. not "session").
+        Thus, simply skip them:
+      */
+      continue;
+    }
+    auto f = [thd, &buf, show, node](const System_variable_tracker &,
+                                     sys_var *var) {
       show->name = var->name.str;
       show->value = (char *)var;
 
-      value = get_one_variable(thd, show, OPT_SESSION, show->type, nullptr,
-                               &charset, val_buf, &val_length);
+      char val_buf[1024];
+      size_t val_length;
+      const CHARSET_INFO *charset;
+      const char *value =
+          get_one_variable(thd, show, OPT_SESSION, show->type, nullptr,
+                           &charset, val_buf, &val_length);
 
-      length = net_length_size(node->m_sysvar_name.length) +
-               node->m_sysvar_name.length + net_length_size(val_length) +
-               val_length;
-
-      to = (uchar *)buf.prep_append(net_length_size(length) + 1, EXTRA_ALLOC);
+      size_t length = net_length_size(node->m_sysvar_name.length) +
+                      node->m_sysvar_name.length + net_length_size(val_length) +
+                      val_length;
+      /* allocate 1 bytes more for session state type. */
+      uchar *to =
+          (uchar *)buf.prep_append(net_length_size(length) + 1, EXTRA_ALLOC);
 
       /* Session state type (SESSION_TRACK_SYSTEM_VARIABLES) */
       to = net_store_length(to, (ulonglong)SESSION_TRACK_SYSTEM_VARIABLES);
@@ -699,6 +758,11 @@ bool Session_sysvars_tracker::store(THD *thd, String &buf) {
 
       /* System variable's value (length-encoded string). */
       store_lenenc_string(buf, value, val_length);
+    };
+    if (System_variable_tracker::make_tracker(
+            to_string_view(node->m_sysvar_name))
+            .access_system_variable(thd, f, Suppress_not_found_error::YES)) {
+      continue;
     }
   }
 
@@ -715,17 +779,14 @@ bool Session_sysvars_tracker::store(THD *thd, String &buf) {
 */
 
 void Session_sysvars_tracker::mark_as_changed(THD *thd,
-                                              LEX_CSTRING *tracked_item_name) {
-  DBUG_ASSERT(tracked_item_name->str);
+                                              LEX_CSTRING tracked_item_name) {
+  assert(tracked_item_name.str);
   sysvar_node_st *node = nullptr;
-  LEX_CSTRING tmp;
-  tmp.str = tracked_item_name->str;
-  tmp.length = tracked_item_name->length;
   /*
     Check if the specified system variable is being tracked, if so
     mark it as changed and also set the class's m_changed flag.
   */
-  if ((node = orig_list->search(node, tmp))) {
+  if ((node = orig_list->search(node, tracked_item_name))) {
     node->m_changed = true;
     m_changed = true;
     /* do not cache the statement when there is change in session state */
@@ -804,7 +865,6 @@ bool Current_schema_tracker::store(THD *thd, String &buf) {
 
   /* Session state type (SESSION_TRACK_SCHEMA) */
   to = net_store_length(to, (ulonglong)SESSION_TRACK_SCHEMA);
-
   /* Length of the overall entity. */
   to = net_store_length(to, length);
 
@@ -826,8 +886,9 @@ bool Current_schema_tracker::store(THD *thd, String &buf) {
   @param tracked_item_name Always null (unused).
 */
 
-void Current_schema_tracker::mark_as_changed(
-    THD *thd, LEX_CSTRING *tracked_item_name MY_ATTRIBUTE((unused))) {
+void Current_schema_tracker::mark_as_changed(THD *thd,
+                                             LEX_CSTRING tracked_item_name
+                                             [[maybe_unused]]) {
   m_changed = true;
   thd->lex->safe_to_cache_query = false;
 }
@@ -875,7 +936,7 @@ bool Transaction_state_tracker::update(THD *thd) {
     }
     if (thd->variables.session_track_transaction_info == TX_TRACK_CHISTICS)
       tx_changed |= TX_CHG_CHISTICS;
-    mark_as_changed(thd, nullptr);
+    mark_as_changed(thd, {});
   } else
     m_enabled = false;
 
@@ -978,7 +1039,7 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
            legal and equivalent syntax in MySQL, or START TRANSACTION
            sans options) will re-use any one-shots set up so far
            (with SET before the first transaction started, and with
-           all subsequent STARTs), except for WITH CONSISTANT SNAPSHOT,
+           all subsequent STARTs), except for WITH CONSISTENT SNAPSHOT,
            which will never be chained and only applies when explicitly
            given.
 
@@ -992,7 +1053,7 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
         statement even for a transaction that isn't the first in an
         ongoing chain. Consider
 
-          SET TRANSACTION ISOLATION LEVEL READ UNCOMMITED;
+          SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
           START TRANSACTION READ ONLY, WITH CONSISTENT SNAPSHOT;
           # work
           COMMIT AND CHAIN;
@@ -1000,7 +1061,7 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
         If we switch away at this point, the replay in the new session
         needs to be
 
-          SET TRANSACTION ISOLATION LEVEL READ UNCOMMITED;
+          SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
           START TRANSACTION READ ONLY;
 
         When a transaction ends (COMMIT/ROLLBACK sans CHAIN), all
@@ -1136,13 +1197,13 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
       */
       length += net_length_size(length);  // ... plus that of its length
 
+      /* allocate extra 1 bytes for session state type. */
       uchar *to =
           (uchar *)buf.prep_append(net_length_size(length) + 1, EXTRA_ALLOC);
 
       /* Session state type (SESSION_TRACK_TRANSACTION_CHARACTERISTICS) */
       to = net_store_length(
           (uchar *)to, (ulonglong)SESSION_TRACK_TRANSACTION_CHARACTERISTICS);
-
       /* Length of the overall entity. */
       to = net_store_length((uchar *)to, length);
 
@@ -1160,7 +1221,7 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
   Mark the tracker as changed.
 */
 
-void Transaction_state_tracker::mark_as_changed(THD *, LEX_CSTRING *) {
+void Transaction_state_tracker::mark_as_changed(THD *, LEX_CSTRING) {
   m_changed = true;
 }
 
@@ -1293,7 +1354,7 @@ void Transaction_state_tracker::set_read_flags(THD *thd,
   if (tx_read_flags != flags) {
     tx_read_flags = flags;
     tx_changed |= TX_CHG_CHISTICS;
-    mark_as_changed(thd, nullptr);
+    mark_as_changed(thd, {});
   }
 }
 
@@ -1310,7 +1371,7 @@ void Transaction_state_tracker::set_isol_level(THD *thd,
   if (tx_isol_level != level) {
     tx_isol_level = level;
     tx_changed |= TX_CHG_CHISTICS;
-    mark_as_changed(thd, nullptr);
+    mark_as_changed(thd, {});
   }
 }
 
@@ -1361,15 +1422,14 @@ bool Session_state_change_tracker::update(THD *thd) { return enable(thd); }
 bool Session_state_change_tracker::store(THD *, String &buf) {
   /* since its a boolean tracker length is always 1 */
   const ulonglong length = 1;
-
+  /*
+    format of the payload is as follows:
+    [ tracker type][length] [1 byte flag]
+  */
   uchar *to = (uchar *)buf.prep_append(3, EXTRA_ALLOC);
 
-  /* format of the payload is as follows:
-     [ tracker type] [length] [1 byte flag] */
-
   /* Session state type (SESSION_TRACK_STATE_CHANGE) */
   to = net_store_length(to, (ulonglong)SESSION_TRACK_STATE_CHANGE);
-
   /* Length of the overall entity it is always 1 byte */
   to = net_store_length(to, length);
 
@@ -1387,11 +1447,12 @@ bool Session_state_change_tracker::store(THD *, String &buf) {
 */
 
 void Session_state_change_tracker::mark_as_changed(
-    THD *thd, LEX_CSTRING *tracked_item_name) {
+    THD *thd, LEX_CSTRING tracked_item_name) {
   /* do not send the boolean flag for the tracker itself
      in the OK packet */
-  if (tracked_item_name &&
-      (strncmp(tracked_item_name->str, "session_track_state_change", 26) == 0))
+  if (tracked_item_name.str &&
+      (strncmp(tracked_item_name.str,
+               STRING_WITH_LEN("session_track_state_change")) == 0))
     m_changed = false;
   else {
     m_changed = true;
@@ -1417,6 +1478,20 @@ bool Session_state_change_tracker::is_state_changed() { return m_changed; }
 
 ///////////////////////////////////////////////////////////////////////////////
 
+class Session_transaction_state : public State_tracker {
+ public:
+  /** Constructor */
+  Session_transaction_state() {}
+
+  bool enable(THD *) override { return false; }
+  bool check(THD *, set_var *) override { return false; }
+  bool update(THD *) override { return false; }
+  bool store(THD *, String &) override { return false; }
+  void mark_as_changed(THD *, LEX_CSTRING) override {}
+};
+
+///////////////////////////////////////////////////////////////////////////////
+
 /**
   @brief Initialize session tracker objects.
 
@@ -1433,6 +1508,8 @@ void Session_tracker::init(const CHARSET_INFO *char_set) {
   m_trackers[SESSION_GTIDS_TRACKER] = new (std::nothrow) Session_gtids_tracker;
   m_trackers[TRANSACTION_INFO_TRACKER] =
       new (std::nothrow) Transaction_state_tracker;
+  m_trackers[TRACK_TRANSACTION_STATE] =
+      new (std::nothrow) Session_transaction_state;
 }
 
 void Session_tracker::claim_memory_ownership(bool claim) {
@@ -1613,7 +1690,7 @@ bool Session_gtids_tracker::update(THD *thd) {
          state-change (see reset()).
 
 
-  @param thd           The thd handle.
+  @param thd                The thd handle.
   @param [in,out] buf       Buffer to store the information to.
 
   @return
@@ -1634,9 +1711,9 @@ bool Session_gtids_tracker::store(THD *thd, String &buf) {
   @param tracked_item_name          Always null.
 */
 
-void Session_gtids_tracker::mark_as_changed(THD *thd MY_ATTRIBUTE((unused)),
-                                            LEX_CSTRING *tracked_item_name
-                                                MY_ATTRIBUTE((unused))) {
+void Session_gtids_tracker::mark_as_changed(THD *thd [[maybe_unused]],
+                                            LEX_CSTRING tracked_item_name
+                                            [[maybe_unused]]) {
   m_changed = true;
 }
 
@@ -1660,3 +1737,5 @@ void Session_gtids_tracker::reset() {
   }
   m_changed = false;
 }
+
+///////////////////////////////////////////////////////////////////////////////
