@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MYSQL_VAR_LIB_PB2_SB_1_1362033_1607726061_07_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
-# define YY_MYSQL_VAR_LIB_PB2_SB_1_1362033_1607726061_07_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MYSQL_VAR_LIB_PB2_SB_1_10803630_1679031277_48_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+# define YY_MYSQL_VAR_LIB_PB2_SB_1_10803630_1679031277_48_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -278,7 +278,7 @@ extern int MYSQLdebug;
     IMPORT = 488,
     INDEXES = 489,
     INDEX_SYM = 490,
-    INFILE = 491,
+    INFILE_SYM = 491,
     INITIAL_SIZE_SYM = 492,
     INNER_SYM = 493,
     INOUT_SYM = 494,
@@ -825,11 +825,26 @@ extern int MYSQLdebug;
     SOURCE_TLS_VERSION_SYM = 1186,
     SOURCE_USER_SYM = 1187,
     SOURCE_ZSTD_COMPRESSION_LEVEL_SYM = 1188,
-    KEYWORD_USED_AS_IDENT = 1190,
-    KEYWORD_USED_AS_KEYWORD = 1191,
-    CONDITIONLESS_JOIN = 1192,
-    SUBQUERY_AS_EXPR = 1193,
-    EMPTY_FROM_CLAUSE = 1194
+    ST_COLLECT_SYM = 1189,
+    KEYRING_SYM = 1190,
+    AUTHENTICATION_SYM = 1191,
+    FACTOR_SYM = 1192,
+    FINISH_SYM = 1193,
+    INITIATE_SYM = 1194,
+    REGISTRATION_SYM = 1195,
+    UNREGISTER_SYM = 1196,
+    INITIAL_SYM = 1197,
+    CHALLENGE_RESPONSE_SYM = 1198,
+    GTID_ONLY_SYM = 1199,
+    INTERSECT_SYM = 1200,
+    BULK_SYM = 1201,
+    URL_SYM = 1202,
+    GENERATE_SYM = 1203,
+    KEYWORD_USED_AS_IDENT = 1205,
+    KEYWORD_USED_AS_KEYWORD = 1206,
+    CONDITIONLESS_JOIN = 1207,
+    SUBQUERY_AS_EXPR = 1208,
+    EMPTY_FROM_CLAUSE = 1209
   };
 #endif
 /* Tokens.  */
@@ -1066,7 +1081,7 @@ extern int MYSQLdebug;
 #define IMPORT 488
 #define INDEXES 489
 #define INDEX_SYM 490
-#define INFILE 491
+#define INFILE_SYM 491
 #define INITIAL_SIZE_SYM 492
 #define INNER_SYM 493
 #define INOUT_SYM 494
@@ -1613,11 +1628,26 @@ extern int MYSQLdebug;
 #define SOURCE_TLS_VERSION_SYM 1186
 #define SOURCE_USER_SYM 1187
 #define SOURCE_ZSTD_COMPRESSION_LEVEL_SYM 1188
-#define KEYWORD_USED_AS_IDENT 1190
-#define KEYWORD_USED_AS_KEYWORD 1191
-#define CONDITIONLESS_JOIN 1192
-#define SUBQUERY_AS_EXPR 1193
-#define EMPTY_FROM_CLAUSE 1194
+#define ST_COLLECT_SYM 1189
+#define KEYRING_SYM 1190
+#define AUTHENTICATION_SYM 1191
+#define FACTOR_SYM 1192
+#define FINISH_SYM 1193
+#define INITIATE_SYM 1194
+#define REGISTRATION_SYM 1195
+#define UNREGISTER_SYM 1196
+#define INITIAL_SYM 1197
+#define CHALLENGE_RESPONSE_SYM 1198
+#define GTID_ONLY_SYM 1199
+#define INTERSECT_SYM 1200
+#define BULK_SYM 1201
+#define URL_SYM 1202
+#define GENERATE_SYM 1203
+#define KEYWORD_USED_AS_IDENT 1205
+#define KEYWORD_USED_AS_KEYWORD 1206
+#define CONDITIONLESS_JOIN 1207
+#define SUBQUERY_AS_EXPR 1208
+#define EMPTY_FROM_CLAUSE 1209
 
 /* Value type.  */
 
@@ -1639,4 +1669,4 @@ struct YYLTYPE
 
 int MYSQLparse (class THD *YYTHD, class Parse_tree_root **parse_tree);
 
-#endif /* !YY_MYSQL_VAR_LIB_PB2_SB_1_1362033_1607726061_07_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MYSQL_VAR_LIB_PB2_SB_1_10803630_1679031277_48_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
