@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,6 +26,7 @@
 #define QMGR_H
 
 
+#include <cstring>
 #include <pc.hpp>
 #include <NdbTick.h>
 #include <NdbGetRUsage.h>
@@ -68,6 +69,10 @@
 #define ZARBIT_HANDLING 5
 #define ZSTART_FAILURE_LIMIT 6
 #define ZNOTIFY_STATE_CHANGE 7
+#define ZCHECK_MULTI_TRP_CONNECT 8
+#define ZRESEND_GET_NUM_MULTI_TRP_REQ 9
+#define ZSWITCH_MULTI_TRP 10
+#define ZSEND_TRP_KEEP_ALIVE 11
 
 /* Error Codes ------------------------------*/
 #define ZERRTOOMANY 1101
@@ -222,6 +227,23 @@ public:
     Uint32 hbOrder;
     Phase phase;
 
+    bool m_initial_set_up_multi_trp_done;
+    bool m_is_in_same_nodegroup;
+    bool m_is_multi_trp_setup;
+    bool m_is_get_num_multi_trp_active;
+    bool m_is_activate_trp_ready_for_me;
+    bool m_is_activate_trp_ready_for_other;
+    bool m_is_freeze_thread_completed;
+    bool m_is_ready_to_switch_trp;
+    bool m_is_preparing_switch_trp;
+    bool m_is_using_multi_trp;
+    bool m_set_up_multi_trp_started;
+    Uint8 m_used_num_multi_trps;
+    Uint32 m_num_activated_trps;
+    BlockReference m_multi_trp_blockref;
+    Uint32 m_check_multi_trp_connect_loop_count;
+    Uint32 m_count_multi_trp_ref;
+
     QmgrState sendPrepFailReqStatus;
     QmgrState sendCommitFailReqStatus;
     QmgrState sendPresToStatus;
@@ -231,7 +253,7 @@ public:
     NDB_TICKS m_alloc_timeout;
     Uint16 m_failconf_blocks[QMGR_MAX_FAIL_STATE_BLOCKS];
 
-    NodeRec() { bzero(m_failconf_blocks, sizeof(m_failconf_blocks)); }
+    NodeRec() { std::memset(m_failconf_blocks, 0, sizeof(m_failconf_blocks)); }
   }; /* p2c: size = 52 bytes */
   
   typedef Ptr<NodeRec> NodeRecPtr;
@@ -305,7 +327,7 @@ public:
 
 public:
   Qmgr(Block_context&);
-  virtual ~Qmgr();
+  ~Qmgr() override;
 
 private:
   BLOCK_DEFINES(Qmgr);
@@ -353,9 +375,22 @@ private:
 
   void execDIH_RESTARTREF(Signal* signal);
   void execDIH_RESTARTCONF(Signal* signal);
-  
+
+  void execSET_UP_MULTI_TRP_REQ(Signal*);
+  void execGET_NUM_MULTI_TRP_REQ(Signal*);
+  void execGET_NUM_MULTI_TRP_CONF(Signal*);
+  void execGET_NUM_MULTI_TRP_REF(Signal*);
+  void execFREEZE_ACTION_REQ(Signal*);
+  void execFREEZE_THREAD_CONF(Signal*);
+  void execACTIVATE_TRP_REQ(Signal*);
+  void execACTIVATE_TRP_CONF(Signal*);
+  void execSWITCH_MULTI_TRP_REQ(Signal*);
+  void execSWITCH_MULTI_TRP_CONF(Signal*);
+  void execSWITCH_MULTI_TRP_REF(Signal*);
+
   void execAPI_VERSION_REQ(Signal* signal);
   void execAPI_BROADCAST_REP(Signal* signal);
+  void execTRP_KEEP_ALIVE(Signal* signal);
 
   void execNODE_FAILREP(Signal *);
   void execALLOC_NODEID_REQ(Signal *);
@@ -461,6 +496,9 @@ private:
   void setHbApiDelay(UintR aHbApiDelay);
   void setArbitTimeout(UintR aArbitTimeout);
   void setCCDelay(UintR aCCDelay);
+  void send_trp_keep_alive_start(Signal* signal);
+  void send_trp_keep_alive(Signal* signal);
+  void setTrpKeepAliveSendDelay(Uint32 delay);
 
   // Interface to arbitration module
   void handleArbitStart(Signal* signal);
@@ -483,6 +521,7 @@ private:
   Uint32 count_previously_alive_nodes();
   void computeArbitNdbMask(NodeBitmaskPOD& aMask);
   void computeArbitNdbMask(NdbNodeBitmaskPOD& aMask);
+  void computeBeforeFailNdbMask(NdbNodeBitmaskPOD& aMask);
   void computeNonDiedNdbMask(NdbNodeBitmaskPOD& aMask);
   void reportArbitEvent(Signal* signal, Ndb_logevent_type type,
                         const NodeBitmask mask = NodeBitmask());
@@ -569,11 +608,14 @@ private:
 
   QmgrState ctoStatus;
   bool cHbSent;
+  bool c_keep_alive_send_in_progress;
+  Uint32 c_keepalive_seqnum;
 
   Timer interface_check_timer;
   Timer hb_check_timer;
   Timer hb_send_timer;
   Timer hb_api_timer;
+  Timer ka_send_timer;
 
   Int16 processInfoNodeIndex[MAX_NODES];
   ProcessInfo * receivedProcessInfo;
@@ -612,6 +654,20 @@ private:
                        Uint32 length, 
                        JobBufferLevel jbuf,
                        Uint32 minversion);
+  void get_node_group_mask(Signal*, NdbNodeBitmask&);
+  bool get_num_multi_trps(Signal*, bool&);
+  void create_multi_transporter(NodeId);
+  void connect_multi_transporter(Signal*, NodeId);
+  void check_connect_multi_transporter(Signal*, NodeId);
+  void send_get_num_multi_trp_req(Signal*, NodeId);
+  void send_switch_multi_transporter(Signal*, NodeId, bool);
+  void switch_multi_transporter(Signal*, NodeId);
+  void check_switch_completed(Signal*, NodeId);
+  void check_no_multi_trp(Signal*, NodeId);
+  void check_more_trp_switch_nodes(Signal*);
+  void handle_activate_trp_req(Signal*, Uint32);
+  bool check_all_multi_trp_nodes_connected();
+  bool select_node_id_for_switch(NodeId&, bool);
 
   bool m_micro_gcp_enabled;
 
@@ -624,8 +680,12 @@ private:
 #endif
 
   struct ndb_rusage m_timer_handling_rusage;
-
-  Ndbcntr *c_ndbcntr;
+  Uint32 m_num_multi_trps;
+  Uint32 m_get_num_multi_trps_sent;
+  bool m_initial_set_up_multi_trp_done;
+  Uint32 m_current_switch_multi_trp_node;
+  BlockReference m_ref_set_up_multi_trp_req;
+  Ndbcntr* c_ndbcntr;
 
   Uint32 get_hb_count(Uint32 nodeId) const {
     return globalData.get_hb_count(nodeId);
@@ -639,9 +699,11 @@ private:
 
   void sendReadLocalSysfile(Signal*);
   void execREAD_LOCAL_SYSFILE_CONF(Signal*);
+  bool is_multi_socket_setup_active(Uint32 node_id, bool locked);
+  void complete_multi_trp_setup(Signal*, bool);
+  void dec_get_num_multi_trps_sent(NodeId);
+  void inc_get_num_multi_trps_sent(NodeId);
 };
-
-
 #undef JAM_FILE_ID
 
 #endif
