@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -45,7 +45,6 @@
 
 #define JAM_FILE_ID 329
 
-extern EventLogger * g_eventLogger;
 
 /**
  * Declare the global variables 
@@ -143,7 +142,7 @@ static const char* const jamFileNames[] =
    "ConfigChange.hpp",                   // 7
    "CreateIndx.hpp",                     // 8
    "StartInfo.hpp",                      // 9
-   "GetTableId.hpp",                     // 10
+   "GetTableId.hpp",                     // 10 DELETED FILE
    "NextScan.hpp",                       // 11
    "DihFragCount.hpp",                   // 12
    "CmInit.hpp",                         // 13 DELETED FILE
@@ -457,7 +456,7 @@ static const char* const jamFileNames[] =
    "arrayPoolTest.cpp",                  // 321 DELETED FILE
    "SimBlockList.hpp",                   // 322
    "mt-lock.hpp",                        // 323
-   "rr.cpp",                             // 324
+   "rr.cpp",                             // 324 DELETED FILE
    "testCopy.cpp",                       // 325
    "Ndbinfo.cpp",                        // 326
    "SectionReader.cpp",                  // 327
@@ -511,7 +510,7 @@ static const char* const jamFileNames[] =
    "DbtuxTree.cpp",                      // 375
    "DbtuxProxy.hpp",                     // 376
    "DbtuxMeta.cpp",                      // 377
-   "DbtuxCmp.cpp",                       // 378
+   "DbtuxCmp.cpp",                       // 378 DELETED FILE
    "Cmvmi.hpp",                          // 379
    "Cmvmi.cpp",                          // 380
    "AsyncIoThread.hpp",                  // 381
@@ -523,7 +522,7 @@ static const char* const jamFileNames[] =
    "AsyncFile.cpp",                      // 387
    "AsyncIoThread.cpp",                  // 388
    "AsyncFileTest.cpp",                  // 389 DELETED FILE
-   "MemoryChannel.cpp",                  // 390
+   "MemoryChannel.cpp",                  // 390 DELETED FILE
    "AsyncFile.hpp",                      // 391
    "Filename.hpp",                       // 392
    "Ndbfs.cpp",                          // 393
@@ -649,6 +648,33 @@ static const char* const jamFileNames[] =
    "BlockThreadBitmask.hpp",             // 513
    "SyncThreadViaReqConf.hpp",           // 514
    "TakeOverTcConf.hpp",                 // 515
+   "GetNumMultiTrp.hpp",                 // 516
+   "Dbqlqh.hpp",                         // 517
+   "Dbqlqh.cpp",                         // 518
+   "DbqlqhProxy.hpp",                    // 519
+   "DbqlqhProxy.cpp",                    // 520
+   "Dbqacc.hpp",                         // 521
+   "Dbqacc.cpp",                         // 522
+   "DbqaccProxy.hpp",                    // 523
+   "DbqaccProxy.cpp",                    // 524
+   "Dbqtup.hpp",                         // 525
+   "Dbqtup.cpp",                         // 526
+   "DbqtupProxy.hpp",                    // 527
+   "DbqtupProxy.cpp",                    // 528
+   "Dbqtux.hpp",                         // 529
+   "Dbqtux.cpp",                         // 530
+   "DbqtuxProxy.hpp",                    // 531
+   "DbqtuxProxy.cpp",                    // 532
+   "QBackup.hpp",                        // 533
+   "QBackup.cpp",                        // 534
+   "QBackupProxy.hpp",                   // 535
+   "QBackupProxy.cpp",                   // 536
+   "QRestore.hpp",                       // 537
+   "QRestore.cpp",                       // 538
+   "QRestoreProxy.hpp",                  // 539
+   "QRestoreProxy.cpp",                  // 540
+   "TrpKeepAlive.hpp",                   // 541
+   "TrpKeepAlive.cpp",                   // 542
    };
 
 bool 
