@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -309,6 +309,36 @@ NdbOperation::interpretedUpdateTuple()
   }//if
 }//NdbOperation::interpretedUpdateTuple()
 
+/******************************************************************************
+ * int interpretedWriteTuple();
+ ****************************************************************************/
+int
+NdbOperation::interpretedWriteTuple()
+{
+  NdbTransaction* tNdbCon = theNdbCon;
+  /* Interpreted write only supported in 8.0.30 and above */
+  if (!ndbd_interpreted_write_supported(
+          theNdbCon->getNdb()->getMinDbNodeVersion())) {
+    setErrorCode(4003);
+    return -1;
+  }
+  int tErrorLine = theErrorLine;
+  if (theStatus == Init) {
+    theStatus = OperationDefined;
+    tNdbCon->theSimpleState = 0;
+    theOperationType = WriteRequest;
+    theAI_LenInCurrAI = 25;
+    theLockMode = LM_Exclusive;
+    theErrorLine = tErrorLine++;
+    m_abortOption = AbortOnError;
+    initInterpreter();
+    return 0;
+  } else {
+    setErrorCode(4200);
+    return -1;
+  }//if
+}//NdbOperation::interpretedWriteTuple()
+
 /*****************************************************************************
  * int interpretedDeleteTuple();
  *****************************************************************************/
@@ -383,7 +413,7 @@ NdbRecAttr*
 NdbOperation::getValue_impl(const NdbColumnImpl* tAttrInfo, char* aValue)
 {
   NdbRecAttr* tRecAttr;
-  if ((tAttrInfo != NULL) &&
+  if ((tAttrInfo != nullptr) &&
       (theStatus != Init)){
     if (tAttrInfo->m_storageType == NDB_STORAGETYPE_DISK)
     {
@@ -398,7 +428,7 @@ NdbOperation::getValue_impl(const NdbColumnImpl* tAttrInfo, char* aValue)
 	  ; // Simply continue with getValue
 	} else if (theStatus == ExecInterpretedValue) {
 	  if (insertATTRINFO(Interpreter::EXIT_OK) == -1)
-	    return NULL;
+	    return nullptr;
 	  theInterpretedSize = theTotalCurrAI_Len -
 	    (theInitialReadSize + 5);
 	} else if (theStatus == SetValueInterpreted) {
@@ -406,13 +436,13 @@ NdbOperation::getValue_impl(const NdbColumnImpl* tAttrInfo, char* aValue)
 	    (theInitialReadSize + theInterpretedSize + 5);
 	} else {
 	  setErrorCodeAbort(4230);
-	  return NULL;
+	  return nullptr;
 	}//if
         /* Final read, after running interpreted instructions. */
 	theStatus = FinalGetValue;
       } else {
 	setErrorCodeAbort(4230);
-	return NULL;
+	return nullptr;
       }//if
     }//if
     AttributeHeader ah(tAttrInfo->m_attrId, 0);
@@ -422,24 +452,24 @@ NdbOperation::getValue_impl(const NdbColumnImpl* tAttrInfo, char* aValue)
       /************************************************************************
        * Get a Receive Attribute object and link it into the operation object.
        ***********************************************************************/
-      if((tRecAttr = theReceiver.getValue(tAttrInfo, aValue)) != 0){
+      if((tRecAttr = theReceiver.getValue(tAttrInfo, aValue)) != nullptr){
 	theErrorLine++;
 	return tRecAttr;
       } else {  
 	setErrorCodeAbort(4000);
-	return NULL;
+	return nullptr;
       }
     } else {
-      return NULL;
+      return nullptr;
     }//if insertATTRINFO failure
   } else {
-    if (tAttrInfo == NULL) {
+    if (tAttrInfo == nullptr) {
       setErrorCodeAbort(4004);      
-      return NULL;
+      return nullptr;
     }//if
   }//if
   setErrorCodeAbort(4200);
-  return NULL;
+  return nullptr;
 }
 
 NdbRecAttr*
@@ -456,19 +486,19 @@ NdbOperation::getValue_NdbRecord(const NdbColumnImpl* tAttrInfo, char* aValue)
     For getValue with NdbRecord operations, we just allocate the NdbRecAttr,
     the signal data will be constructed later.
   */
-  if((tRecAttr = theReceiver.getValue(tAttrInfo, aValue)) != 0) {
+  if((tRecAttr = theReceiver.getValue(tAttrInfo, aValue)) != nullptr) {
     theErrorLine++;
     return tRecAttr;
   } else {
     setErrorCodeAbort(4000);
-    return NULL;
+    return nullptr;
   }
 }
 
 /*****************************************************************************
  * int setValue(AttrInfo* tAttrInfo, char* aValue, Uint32 len)
  *
- * Return Value:  Return 0 : SetValue was succesful.
+ * Return Value:  Return 0 : SetValue was successful.
  *                Return -1: In all other case.   
  * Parameters:    tAttrInfo : Attribute object where the attribute 
  *                            info exists.
@@ -481,31 +511,41 @@ NdbOperation::setValue( const NdbColumnImpl* tAttrInfo,
 			const char* aValuePassed)
 {
   DBUG_ENTER("NdbOperation::setValue");
-  DBUG_PRINT("enter", ("col: %s  op:%d  val: 0x%lx",
+  DBUG_PRINT("enter", ("col: %s  op:%d  val: %p",
                        tAttrInfo ? tAttrInfo->m_name.c_str() : "NULL",
-                       theOperationType, (long) aValuePassed));
+                       theOperationType, aValuePassed));
 
   int tReturnCode;
   Uint32 tAttrId;
   Uint32 tData;
   Uint32 tempData[ NDB_MAX_TUPLE_SIZE_IN_WORDS ];
   OperationType tOpType = theOperationType;
-  OperationStatus tStatus = theStatus;
-
   
   if ((tOpType == UpdateRequest) ||
       (tOpType == WriteRequest)) {
     if (theInterpretIndicator == 0) {
-      if (tStatus == SetValue) {
+      if (theStatus == SetValue) {
         ;
       } else {
         setErrorCodeAbort(4234);
         DBUG_RETURN(-1);
       }//if
     } else {
-      if (tStatus == GetValue) {
+      if (theStatus == GetValue) {
         theInitialReadSize = theTotalCurrAI_Len - 5;
-      } else if	(tStatus == ExecInterpretedValue) {
+        //--------------------------------------------------------------------
+        // If the operation is an interpreted write, then transfer the keyinfo
+        // data to Attrinfo since it was skipped at the equal() stage searching
+        // for the row.
+        // For more info check out, NdbOperation::transferKeyInfoToAttrInfo()
+        //--------------------------------------------------------------------
+        if (tOpType == WriteRequest) {
+          if (transferKeyInfoToAttrInfo() != 0) {
+            setErrorCodeAbort(4559);
+            DBUG_RETURN(-1);
+          }
+        }
+      } else if	(theStatus == ExecInterpretedValue) {
 	//--------------------------------------------------------------------
 	// We insert an exit from interpretation since we are now starting 
 	// to set values in the tuple by setValue.
@@ -515,7 +555,19 @@ NdbOperation::setValue( const NdbColumnImpl* tAttrInfo,
 	}
         theInterpretedSize = theTotalCurrAI_Len - 
           (theInitialReadSize + 5);
-      } else if (tStatus == SetValueInterpreted) {
+        //--------------------------------------------------------------------
+        // If the operation is an interpreted write, then transfer the keyinfo
+        // data to Attrinfo since it was skipped at the equal() stage searching
+        // for the row.
+        // For more info check out, NdbOperation::transferKeyInfoToAttrInfo()
+        //--------------------------------------------------------------------
+        if (tOpType == WriteRequest) {
+          if (transferKeyInfoToAttrInfo() != 0) {
+            setErrorCodeAbort(4559);
+            DBUG_RETURN(-1);
+          }
+        }
+      } else if (theStatus == SetValueInterpreted) {
         ; // Simply continue adding new setValue
       } else {
 	//--------------------------------------------------------------------
@@ -548,7 +600,7 @@ NdbOperation::setValue( const NdbColumnImpl* tAttrInfo,
     setErrorCodeAbort(4108);
     DBUG_RETURN(-1);
   }//if
-  if (tAttrInfo == NULL) {
+  if (tAttrInfo == nullptr) {
     setErrorCodeAbort(4004);      
     DBUG_RETURN(-1);
   }//if
@@ -568,7 +620,7 @@ NdbOperation::setValue( const NdbColumnImpl* tAttrInfo,
     m_flags &= ~Uint8(OF_NO_DISK);
   }
   const char *aValue = aValuePassed; 
-  if (aValue == NULL) {
+  if (aValue == nullptr) {
     if (tAttrInfo->m_nullable) {
       AttributeHeader ah(tAttrId, 0);
       ah.setNULL();
@@ -617,13 +669,13 @@ NdbOperation::setValue( const NdbColumnImpl* tAttrInfo,
    * If it is not aligned then we start by copying the value to tempData and 
    * use this as aValue instead.
    *************************************************************************/
-  
-  tReturnCode = insertATTRINFOloop((Uint32*)aValue, sizeInWords);
+
+  tReturnCode = insertATTRINFOloop((const Uint32*)aValue, sizeInWords);
   if (tReturnCode == -1) {
     DBUG_RETURN(tReturnCode);
   }//if
   if (bitsInLastWord != 0) {
-    tData = *(Uint32*)(aValue + sizeInWords*4);
+    tData = *(const Uint32*)(aValue + sizeInWords * 4);
     tData = convertEndian(tData);
     tData = tData & ((1 << bitsInLastWord) - 1);
     tData = convertEndian(tData);
@@ -663,6 +715,7 @@ NdbOperation::setAnyValue(Uint32 any_value)
     }
   }
   // Fall through - let setValue call set error
+  [[fallthrough]];
   default:
     return setValue(impl, (const char *)&any_value);
   }
@@ -685,8 +738,8 @@ NdbBlob*
 NdbOperation::getBlobHandle(NdbTransaction* aCon, const NdbColumnImpl* tAttrInfo)
 {
   NdbBlob* tBlob = theBlobList;
-  NdbBlob* tLastBlob = NULL;
-  while (tBlob != NULL) {
+  NdbBlob* tLastBlob = nullptr;
+  while (tBlob != nullptr) {
     if (tBlob->theColumn == tAttrInfo)
       return tBlob;
     tLastBlob = tBlob;
@@ -704,7 +757,7 @@ NdbOperation::getBlobHandle(NdbTransaction* aCon, const NdbColumnImpl* tAttrInfo
   if (m_attribute_record)
   {
     setErrorCodeAbort(4288);
-    return NULL;
+    return nullptr;
   }
 
   /* Check key fully defined for key operations */
@@ -723,33 +776,33 @@ NdbOperation::getBlobHandle(NdbTransaction* aCon, const NdbColumnImpl* tAttrInfo
     /* Unexpected state to be obtaining Blob handle */
     /* Invalid usage of blob attribute */
     setErrorCodeAbort(4264);
-    return NULL;
+    return nullptr;
   }
   }
 
   tBlob = theNdb->getNdbBlob();
-  if (tBlob == NULL)
-    return NULL;
+  if (tBlob == nullptr)
+    return nullptr;
   if (tBlob->atPrepare(aCon, this, tAttrInfo) == -1) {
     theNdb->releaseNdbBlob(tBlob);
-    return NULL;
+    return nullptr;
   }
-  if (tLastBlob == NULL)
+  if (tLastBlob == nullptr)
     theBlobList = tBlob;
   else
     tLastBlob->theNext = tBlob;
-  tBlob->theNext = NULL;
+  tBlob->theNext = nullptr;
   theNdbCon->theBlobFlag = true;
   theNdbCon->m_userDefinedBlobOps = true;
   return tBlob;
 }
 
 /* const variant of getBlobHandle - only returns existing blob handles */
-NdbBlob*
-NdbOperation::getBlobHandle(NdbTransaction* aCon, const NdbColumnImpl* tAttrInfo) const
+NdbBlob* NdbOperation::getBlobHandle(NdbTransaction* /*aCon*/,
+                                     const NdbColumnImpl* tAttrInfo) const
 {
   NdbBlob* tBlob = theBlobList;
-  while (tBlob != NULL) {
+  while (tBlob != nullptr) {
     if (tBlob->theColumn == tAttrInfo)
       return tBlob;
     tBlob = tBlob->theNext;
@@ -760,7 +813,7 @@ NdbOperation::getBlobHandle(NdbTransaction* aCon, const NdbColumnImpl* tAttrInfo
     or NdbRecAttr
   */
   setErrorCodeAbort(4288);
-  return NULL;
+  return nullptr;
 }
 
 /*
@@ -786,15 +839,15 @@ NdbOperation::linkInBlobHandle(NdbTransaction *aCon,
   int res;
 
   NdbBlob *bh= theNdb->getNdbBlob();
-  if (bh == NULL)
-    return NULL;
+  if (bh == nullptr)
+    return nullptr;
 
   if (theOperationType == OpenScanRequest ||
       theOperationType == OpenRangeScanRequest)
   {
     res= bh->atPrepareNdbRecordScan(aCon, this, column);
   }
-  else if (m_key_record == NULL)
+  else if (m_key_record == nullptr)
   {
     /* This means that we have a scan take-over operation, and we should
        obtain the key from KEYINFO20 data.
@@ -809,14 +862,14 @@ NdbOperation::linkInBlobHandle(NdbTransaction *aCon,
   if (res == -1)
   {
     theNdb->releaseNdbBlob(bh);
-    return NULL;
+    return nullptr;
   }
   if (lastPtr)
     lastPtr->theNext= bh;
   else
     theBlobList= bh;
   lastPtr= bh;
-  bh->theNext= NULL;
+  bh->theNext= nullptr;
   theNdbCon->theBlobFlag= true;
   theNdbCon->m_userDefinedBlobOps = true;
 
@@ -834,7 +887,7 @@ int
 NdbOperation::getBlobHandlesNdbRecord(NdbTransaction* aCon, 
                                       const Uint32 * m_read_mask)
 {
-  NdbBlob *lastBlob= NULL;
+  NdbBlob *lastBlob= nullptr;
 
   for (Uint32 i= 0; i<m_attribute_record->noOfColumns; i++)
   {
@@ -848,19 +901,19 @@ NdbOperation::getBlobHandlesNdbRecord(NdbTransaction* aCon,
       continue;
 
     const NdbColumnImpl *tableColumn= m_currentTable->getColumn(attrId);
-    assert(tableColumn != NULL);
+    assert(tableColumn != nullptr);
 
     NdbBlob *bh= linkInBlobHandle(aCon, tableColumn, lastBlob);
-    if (bh == NULL)
+    if (bh == nullptr)
       return -1;
 
     if (theOperationType == ReadRequest || theOperationType == ReadExclusive)
     {
       /*
        * For read request, it is safe to cast away const-ness for the
-       * m_attribute_row.
+       * m_attribute_row. // TODO: why?
        */
-      memcpy((char *)&m_attribute_row[col->offset], &bh, sizeof(bh));
+      memcpy(const_cast<char*>(&m_attribute_row[col->offset]), &bh, sizeof(bh));
     }
   }
 
@@ -878,14 +931,14 @@ NdbOperation::getBlobHandlesNdbRecordDelete(NdbTransaction* aCon,
                                             bool checkReadSet,
                                             const Uint32 * m_read_mask)
 {
-  NdbBlob *lastBlob= NULL;
+  NdbBlob *lastBlob= nullptr;
 
   assert(theOperationType == DeleteRequest);
 
   for (Uint32 i= 0; i < m_currentTable->m_columns.size(); i++)
   {
     const NdbColumnImpl* c= m_currentTable->m_columns[i];
-    assert(c != 0);
+    assert(c != nullptr);
     if (!c->getBlobType())
       continue;
 
@@ -899,7 +952,7 @@ NdbOperation::getBlobHandlesNdbRecordDelete(NdbTransaction* aCon,
     }
 
     NdbBlob *bh= linkInBlobHandle(aCon, c, lastBlob);
-    if (bh == NULL)
+    if (bh == nullptr)
       return -1;
   }
 
@@ -911,8 +964,8 @@ NdbOperation::getVarValue(const NdbColumnImpl* tAttrInfo,
                           char* aBareValue, Uint16* aLenLoc)
 {
   NdbRecAttr* ra = getValue(tAttrInfo, aBareValue);
-  if (ra != NULL) {
-    assert(aLenLoc != NULL);
+  if (ra != nullptr) {
+    assert(aLenLoc != nullptr);
     ra->m_getVarValue = aLenLoc;
   }
   return ra;
@@ -941,7 +994,7 @@ NdbOperation::setVarValue(const NdbColumnImpl* tAttrInfo,
 /****************************************************************************
  * int insertATTRINFO( Uint32 aData );
  *
- * Return Value:   Return 0 : insertATTRINFO was succesful.
+ * Return Value:   Return 0 : insertATTRINFO was successful.
  *                 Return -1: In all other case.   
  * Parameters:     aData: the data to insert into ATTRINFO.
  * Remark:         Puts the the data into either TCKEYREQ signal or 
@@ -960,16 +1013,16 @@ NdbOperation::insertATTRINFO( Uint32 aData )
     NdbApiSignal* tFirstAttrinfo = theFirstATTRINFO;
     tAI_LenInCurrAI = 3;
     tSignal = tNdb->getSignal();
-    if (tSignal != NULL) {
+    if (tSignal != nullptr) {
       tSignal->setSignal(m_attrInfoGSN, refToBlock(theNdbCon->m_tcRef));
       tAttrPtr = &tSignal->getDataPtrSend()[3];
-      if (tFirstAttrinfo == NULL) {
-        tSignal->next(NULL);
+      if (tFirstAttrinfo == nullptr) {
+        tSignal->next(nullptr);
         theFirstATTRINFO = tSignal;
         theCurrentATTRINFO = tSignal;
       } else {
         NdbApiSignal* tCurrentAttrinfoBeforeUpdate = theCurrentATTRINFO;
-        tSignal->next(NULL);
+        tSignal->next(nullptr);
         theCurrentATTRINFO = tSignal;
         tCurrentAttrinfoBeforeUpdate->next(tSignal);
       }//if
@@ -995,7 +1048,7 @@ insertATTRINFO_error1:
 /*****************************************************************************
  * int insertATTRINFOloop(Uint32* aDataPtr, Uint32 aLength );
  *
- * Return Value:  Return 0 : insertATTRINFO was succesful.
+ * Return Value:  Return 0 : insertATTRINFO was successful.
  *                Return -1: In all other case.   
  * Parameters:    aDataPtr: Pointer to the data to insert into ATTRINFO.
  *                aLength: Length of data to be copied
@@ -1017,16 +1070,16 @@ NdbOperation::insertATTRINFOloop(const Uint32* aDataPtr,
       NdbApiSignal* tFirstAttrinfo = theFirstATTRINFO;
       tAI_LenInCurrAI = 3;
       tSignal = tNdb->getSignal();
-      if (tSignal != NULL) {
+      if (tSignal != nullptr) {
         tSignal->setSignal(m_attrInfoGSN, refToBlock(theNdbCon->m_tcRef));
         tAttrPtr = &tSignal->getDataPtrSend()[3];
-        if (tFirstAttrinfo == NULL) {
-          tSignal->next(NULL);
+        if (tFirstAttrinfo == nullptr) {
+          tSignal->next(nullptr);
           theFirstATTRINFO = tSignal;
           theCurrentATTRINFO = tSignal;
         } else {
           NdbApiSignal* tCurrentAttrinfoBeforeUpdate = theCurrentATTRINFO;
-          tSignal->next(NULL);
+          tSignal->next(nullptr);
           theCurrentATTRINFO = tSignal;
           tCurrentAttrinfoBeforeUpdate->next(tSignal);
         }//if
@@ -1091,7 +1144,7 @@ NdbOperation::prepareGetLockHandleNdbRecord()
    * when the OO_LOCKHANDLE flag is set on an NdbRecord
    * operation.
    */
-  assert(theLockHandle == NULL);
+  assert(theLockHandle == nullptr);
   theLockHandle = theNdbCon->getLockHandle();
   if (!theLockHandle)
   {
@@ -1171,7 +1224,7 @@ NdbOperation::handleOperationOptions (const OperationType type,
     return 4297;
   }
 
-  bool isScanTakeoverOp = (op->m_key_record == NULL); 
+  bool isScanTakeoverOp = (op->m_key_record == nullptr); 
   
   if (opts->optionsPresent & OperationOptions::OO_ABORTOPTION)
   {
@@ -1196,7 +1249,7 @@ NdbOperation::handleOperationOptions (const OperationType type,
   if ((opts->optionsPresent & OperationOptions::OO_GETVALUE) &&
       (opts->numExtraGetValues > 0))
   {
-    if (opts->extraGetValues == NULL)
+    if (opts->extraGetValues == nullptr)
     {
       // Incorrect combination of OperationOptions optionsPresent, 
       // extraGet/SetValues ptr and numExtraGet/SetValues
@@ -1204,10 +1257,11 @@ NdbOperation::handleOperationOptions (const OperationType type,
     }
 
     // Only certain operation types allow extra GetValues
-    // Update could be made to support it in future
     if (type == ReadRequest ||
         type == ReadExclusive ||
-        type == DeleteRequest)
+        type == DeleteRequest ||
+        type == UpdateRequest ||
+        type == WriteRequest)
     {
       // Could be readTuple(), or lockCurrentTuple().
       // We perform old-school NdbRecAttr reads on
@@ -1217,9 +1271,9 @@ NdbOperation::handleOperationOptions (const OperationType type,
         GetValueSpec *pvalSpec 
           = &(opts->extraGetValues[i]);
 
-        pvalSpec->recAttr=NULL;
+        pvalSpec->recAttr=nullptr;
 
-        if (pvalSpec->column == NULL)
+        if (pvalSpec->column == nullptr)
         {
           // Column is NULL in Get/SetValueSpec structure
           return 4295;
@@ -1229,7 +1283,7 @@ NdbOperation::handleOperationOptions (const OperationType type,
           op->getValue_NdbRecord(&NdbColumnImpl::getImpl(*pvalSpec->column),
                                  (char *) pvalSpec->appStorage);
         
-        if (pra == NULL)
+        if (pra == nullptr)
         {
           return -1;
         }
@@ -1242,12 +1296,6 @@ NdbOperation::handleOperationOptions (const OperationType type,
       // Bad operation type for GetValue
       switch (type)
       {
-      case WriteRequest : 
-      case UpdateRequest :
-      {
-        return 4502;
-        // GetValue not allowed in Update operation
-      }
       case InsertRequest :
       {
         return 4503;
@@ -1263,7 +1311,7 @@ NdbOperation::handleOperationOptions (const OperationType type,
   if ((opts->optionsPresent & OperationOptions::OO_SETVALUE) &&
       (opts->numExtraSetValues > 0))
   {
-    if (opts->extraSetValues == NULL)
+    if (opts->extraSetValues == nullptr)
     {
       // Incorrect combination of OperationOptions optionsPresent, 
       // extraGet/SetValues ptr and numExtraGet/SetValues
@@ -1283,7 +1331,7 @@ NdbOperation::handleOperationOptions (const OperationType type,
         const NdbDictionary::Column *pcol=opts->extraSetValues[i].column;
         const void *pvalue=opts->extraSetValues[i].value;
 
-        if (pcol == NULL)
+        if (pcol == nullptr)
         {
           // Column is NULL in Get/SetValueSpec structure
           return 4295;
@@ -1297,7 +1345,7 @@ NdbOperation::handleOperationOptions (const OperationType type,
           return 4202;
         }
 
-        if (pvalue == NULL)
+        if (pvalue == nullptr)
         {
           if (!pcol->getNullable())
           {
@@ -1330,7 +1378,7 @@ NdbOperation::handleOperationOptions (const OperationType type,
   if (opts->optionsPresent & OperationOptions::OO_PARTITION_ID)
   {
     /* Should not have any blobs defined at this stage */
-    assert(op->theBlobList == NULL);
+    assert(op->theBlobList == nullptr);
 
     /* Not allowed for scan takeover ops */
     if (unlikely(isScanTakeoverOp))
@@ -1345,7 +1393,7 @@ NdbOperation::handleOperationOptions (const OperationType type,
      */
     if (unlikely( ! (((op->m_attribute_record->flags & 
                        NdbRecord::RecHasUserDefinedPartitioning) &&
-                      (op->m_key_record->table->m_index == NULL)) ||
+                      (op->m_key_record->table->m_index == nullptr)) ||
                      (type == UnlockRequest))))
     {
       /* Explicit partitioning info not allowed for table and operation*/
@@ -1361,16 +1409,24 @@ NdbOperation::handleOperationOptions (const OperationType type,
     if (! ((type == ReadRequest)   ||
            (type == ReadExclusive) ||
            (type == UpdateRequest) ||
-           (type == DeleteRequest)))
+           (type == DeleteRequest) ||
+           (type == WriteRequest)))
       /* NdbInterpretedCode not supported for operation type */
       return 4539;
-    
+
+    /* Interpreted write onlys supported in 8.0.30 and above */
+    if (type == WriteRequest &&
+        !ndbd_interpreted_write_supported(
+            op->theNdbCon->getNdb()->getMinDbNodeVersion())) {
+      return 4003;
+    }
+
     /* Check the program's for the same table as the
      * operation, within a major version number
      * Perhaps NdbInterpretedCode should not contain the table
      */
     const NdbDictionary::Table* codeTable= opts->interpretedCode->getTable();
-    if (codeTable != NULL)
+    if (codeTable != nullptr)
     {
       NdbTableImpl* impl= &NdbTableImpl::getImpl(*codeTable);
       
